@@ -36,7 +36,7 @@
 #include "misc.hh"
 #include <vector>
 #include <sstream>
-#include <errno.h>
+#include <cerrno>
 #include <cstring>
 #include <iostream>
 #include <sys/types.h>
@@ -45,12 +45,14 @@
 #include <poll.h>
 #include <iomanip>
 #include <netinet/tcp.h>
+#include <optional>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include "pdnsexception.hh"
 #include <sys/types.h>
 #include <boost/algorithm/string.hpp>
+#include <boost/format.hpp>
 #include "iputils.hh"
 #include "dnsparser.hh"
 #include <sys/types.h>
@@ -197,6 +199,32 @@ size_t writen2WithTimeout(int fd, const void * buffer, size_t len, const struct
   return len;
 }
 
+auto pdns::getMessageFromErrno(const int errnum) -> std::string
+{
+  const size_t errLen = 2048;
+  std::string errMsgData{};
+  errMsgData.resize(errLen);
+
+  const char* errMsg = nullptr;
+#ifdef STRERROR_R_CHAR_P
+  errMsg = strerror_r(errnum, errMsgData.data(), errMsgData.length());
+#else
+  // This can fail, and when it does, it sets errno. We ignore that and
+  // set our own error message instead.
+  int res = strerror_r(errnum, errMsgData.data(), errMsgData.length());
+  errMsg = errMsgData.c_str();
+  if (res != 0) {
+    errMsg = "Unknown (the exact error could not be retrieved)";
+  }
+#endif
+
+  // We make a copy here because `strerror_r()` might return a static
+  // immutable buffer for an error message. The copy shouldn't be
+  // critical though, we're on the bailout/error-handling path anyways.
+  std::string message{errMsg};
+  return message;
+}
+
 string nowTime()
 {
   time_t now = time(nullptr);
@@ -204,8 +232,10 @@ string nowTime()
   localtime_r(&now, &tm);
   char buffer[30];
   // YYYY-mm-dd HH:MM:SS TZOFF
-  strftime(buffer, sizeof(buffer), "%F %T %z", &tm);
-  buffer[sizeof(buffer)-1] = '\0';
+  size_t ret = strftime(buffer, sizeof(buffer), "%F %T %z", &tm);
+  if (ret == 0) {
+    buffer[0] = '\0';
+  }
   return string(buffer);
 }
 
@@ -280,29 +310,32 @@ bool stripDomainSuffix(string *qname, const string &domain)
   return true;
 }
 
-static void parseService4(const string &descr, ServiceTuple &st)
+static void parseService4(const string& descr, ServiceTuple& st)
 {
-  vector<string>parts;
-  stringtok(parts,descr,":");
-  if(parts.empty())
-    throw PDNSException("Unable to parse '"+descr+"' as a service");
-  st.host=parts[0];
-  if(parts.size()>1)
-    st.port=pdns_stou(parts[1]);
+  vector<string> parts;
+  stringtok(parts, descr, ":");
+  if (parts.empty()) {
+    throw PDNSException("Unable to parse '" + descr + "' as a service");
+  }
+  st.host = parts[0];
+  if (parts.size() > 1) {
+    pdns::checked_stoi_into(st.port, parts[1]);
+  }
 }
 
-static void parseService6(const string &descr, ServiceTuple &st)
+static void parseService6(const string& descr, ServiceTuple& st)
 {
-  string::size_type pos=descr.find(']');
-  if(pos == string::npos)
-    throw PDNSException("Unable to parse '"+descr+"' as an IPv6 service");
+  string::size_type pos = descr.find(']');
+  if (pos == string::npos) {
+    throw PDNSException("Unable to parse '" + descr + "' as an IPv6 service");
+  }
 
-  st.host=descr.substr(1, pos-1);
-  if(pos + 2 < descr.length())
-    st.port=pdns_stou(descr.substr(pos+2));
+  st.host = descr.substr(1, pos - 1);
+  if (pos + 2 < descr.length()) {
+    pdns::checked_stoi_into(st.port, descr.substr(pos + 2));
+  }
 }
 
-
 void parseService(const string &descr, ServiceTuple &st)
 {
   if(descr.empty())
@@ -671,43 +704,47 @@ string stripDot(const string& dom)
   return dom.substr(0,dom.size()-1);
 }
 
-
-
 int makeIPv6sockaddr(const std::string& addr, struct sockaddr_in6* ret)
 {
-  if(addr.empty())
+  if (addr.empty()) {
     return -1;
+  }
+
   string ourAddr(addr);
-  bool portSet = false;
-  unsigned int port;
-  if(addr[0]=='[') { // [::]:53 style address
+  std::optional<uint16_t> port = std::nullopt;
+
+  if (addr[0] == '[') { // [::]:53 style address
     string::size_type pos = addr.find(']');
-    if(pos == string::npos)
+    if (pos == string::npos) {
       return -1;
-    ourAddr.assign(addr.c_str() + 1, pos-1);
+    }
+
+    ourAddr.assign(addr.c_str() + 1, pos - 1);
     if (pos + 1 != addr.size()) { // complete after ], no port specified
-      if (pos + 2 > addr.size() || addr[pos+1]!=':')
+      if (pos + 2 > addr.size() || addr[pos + 1] != ':') {
         return -1;
-      try {
-        port = pdns_stou(addr.substr(pos+2));
-        portSet = true;
       }
-      catch(const std::out_of_range&) {
+
+      try {
+        auto tmpPort = pdns::checked_stoi<uint16_t>(addr.substr(pos + 2));
+        port = std::make_optional(tmpPort);
+      }
+      catch (const std::out_of_range&) {
         return -1;
       }
     }
   }
-  ret->sin6_scope_id=0;
-  ret->sin6_family=AF_INET6;
 
-  if(inet_pton(AF_INET6, ourAddr.c_str(), (void*)&ret->sin6_addr) != 1) {
-    struct addrinfo* res;
-    struct addrinfo hints;
-    memset(&hints, 0, sizeof(hints));
+  ret->sin6_scope_id = 0;
+  ret->sin6_family = AF_INET6;
 
-    hints.ai_family = AF_INET6;
+  if (inet_pton(AF_INET6, ourAddr.c_str(), (void*)&ret->sin6_addr) != 1) {
+    struct addrinfo hints{};
+    std::memset(&hints, 0, sizeof(struct addrinfo));
     hints.ai_flags = AI_NUMERICHOST;
+    hints.ai_family = AF_INET6;
 
+    struct addrinfo* res = nullptr;
     // getaddrinfo has anomalous return codes, anything nonzero is an error, positive or negative
     if (getaddrinfo(ourAddr.c_str(), nullptr, &hints, &res) != 0) {
       return -1;
@@ -717,11 +754,8 @@ int makeIPv6sockaddr(const std::string& addr, struct sockaddr_in6* ret)
     freeaddrinfo(res);
   }
 
-  if(portSet) {
-    if(port > 65535)
-      return -1;
-
-    ret->sin6_port = htons(port);
+  if (port.has_value()) {
+    ret->sin6_port = htons(*port);
   }
 
   return 0;
@@ -794,12 +828,10 @@ bool readFileIfThere(const char* fname, std::string* line)
 {
   line->clear();
   auto fp = std::unique_ptr<FILE, int(*)(FILE*)>(fopen(fname, "r"), fclose);
-  if(!fp)
+  if (!fp) {
     return false;
-  stringfgets(fp.get(), *line);
-  fp.reset();
-
-  return true;
+  }
+  return stringfgets(fp.get(), *line);
 }
 
 Regex::Regex(const string &expr)
@@ -1111,31 +1143,40 @@ bool setCloseOnExec(int sock)
   return true;
 }
 
-string getMACAddress(const ComboAddress& ca)
+int getMACAddress(const ComboAddress& ca, char* dest, size_t len)
 {
-  string ret;
 #ifdef __linux__
   ifstream ifs("/proc/net/arp");
-  if(!ifs)
-    return ret;
+  if (len < 6) {
+    return EINVAL;
+  }
+  if (!ifs) {
+    return EIO;
+  }
   string line;
-  string match=ca.toString()+' ';
+  string match = ca.toString() + ' ';
   while(getline(ifs, line)) {
     if(boost::starts_with(line, match)) {
       vector<string> parts;
       stringtok(parts, line, " \n\t\r");
-      if(parts.size() < 4)
-        return ret;
-      unsigned int tmp[6];
-      if (sscanf(parts[3].c_str(), "%02x:%02x:%02x:%02x:%02x:%02x", tmp, tmp+1, tmp+2, tmp+3, tmp+4, tmp+5) != 6) {
-        return ret;
+      if (parts.size() < 4)
+        return ENOENT;
+      if (sscanf(parts[3].c_str(), "%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx", dest, dest+1, dest+2, dest+3, dest+4, dest+5) != 6) {
+        return ENOENT;
       }
-      for(unsigned int i : tmp)
-        ret.append(1, (char)i);
-      return ret;
+      return 0;
     }
   }
 #endif
+  return ENOENT;
+}
+string getMACAddress(const ComboAddress& ca)
+{
+  string ret;
+  char tmp[6];
+  if (getMACAddress(ca, tmp, sizeof(tmp)) == 0) {
+    ret.append(tmp, sizeof(tmp));
+  }
   return ret;
 }
 
@@ -1336,7 +1377,7 @@ uint64_t getOpenFileDescriptors(const std::string&)
 {
 #ifdef __linux__
   DIR* dirhdl=opendir(("/proc/"+std::to_string(getpid())+"/fd/").c_str());
-  if(!dirhdl) 
+  if(!dirhdl)
     return 0;
 
   struct dirent *entry;
@@ -1344,7 +1385,7 @@ uint64_t getOpenFileDescriptors(const std::string&)
   while((entry = readdir(dirhdl))) {
     uint32_t num;
     try {
-      num = pdns_stou(entry->d_name);
+      pdns::checked_stoi_into(num, entry->d_name);
     } catch (...) {
       continue; // was not a number.
     }
@@ -1373,7 +1414,7 @@ uint64_t getRealMemoryUsage(const std::string&)
   ifs >> size >> resident >> shared >> text >> lib >> data;
 
   // We used to use "data" here, but it proves unreliable and even is marked "broken"
-  // in https://www.kernel.org/doc/html/latest/filesystems/proc.html 
+  // in https://www.kernel.org/doc/html/latest/filesystems/proc.html
   return resident * getpagesize();
 #else
   struct rusage ru;
@@ -1422,7 +1463,7 @@ double DiffTime(const struct timespec& first, const struct timespec& second)
 {
   int seconds=second.tv_sec - first.tv_sec;
   int nseconds=second.tv_nsec - first.tv_nsec;
-  
+
   if(nseconds < 0) {
     seconds-=1;
     nseconds+=1000000000;
@@ -1434,7 +1475,7 @@ double DiffTime(const struct timeval& first, const struct timeval& second)
 {
   int seconds=second.tv_sec - first.tv_sec;
   int useconds=second.tv_usec - first.tv_usec;
-  
+
   if(useconds < 0) {
     seconds-=1;
     useconds+=1000000;
@@ -1500,25 +1541,6 @@ gid_t strToGID(const string &str)
   return result;
 }
 
-unsigned int pdns_stou(const std::string& str, size_t * idx, int base)
-{
-  if (str.empty()) return 0; // compatibility
-  unsigned long result;
-  try {
-    result = std::stoul(str, idx, base);
-  }
-  catch(std::invalid_argument& e) {
-    throw std::invalid_argument(string(e.what()) + "; (invalid argument during std::stoul); data was \""+str+"\"");
-  }
-  catch(std::out_of_range& e) {
-    throw std::out_of_range(string(e.what()) + "; (out of range during std::stoul); data was \""+str+"\"");
-  }
-  if (result > std::numeric_limits<unsigned int>::max()) {
-    throw std::out_of_range("stoul returned result out of unsigned int range; data was \""+str+"\"");
-  }
-  return static_cast<unsigned int>(result);
-}
-
 bool isSettingThreadCPUAffinitySupported()
 {
 #ifdef HAVE_PTHREAD_SETAFFINITY_NP
