@@ -24,6 +24,8 @@
 #endif
 
 #include <boost/tokenizer.hpp>
+#include <boost/format.hpp>
+
 #include "namespaces.hh"
 #include "ws-api.hh"
 #include "json.hh"
@@ -97,7 +99,10 @@ static Json getServerDetail() {
     { "daemon_type", productTypeApiType() },
     { "version", getPDNSVersion() },
     { "config_url", "/api/v1/servers/localhost/config{/config_setting}" },
-    { "zones_url", "/api/v1/servers/localhost/zones{/zone}" }
+    { "zones_url", "/api/v1/servers/localhost/zones{/zone}" },
+#ifndef RECURSOR
+    { "autoprimaries_url", "/api/v1/servers/localhost/autoprimaries{/autoprimary}" }
+#endif
   };
 }
 
